Package: GRaNIE
Title: GRaNIE: Reconstruction cell type specific gene regulatory
        networks including enhancers using single-cell or bulk
        chromatin accessibility and RNA-seq data
Version: 1.15.0
Encoding: UTF-8
Authors@R: c(person("Christian", "Arnold", email =
        "chrarnold@web.de", role = c("cre","aut")),
        person("Judith", "Zaugg", email =
        "judith.zaugg@embl.de", role = c("aut")),
        person("Rim", "Moussa", email =
        "rim.moussa01@gmail.com", role = "aut"),
        person("Armando", "Reyes-Palomares", email =
        "armandorp@gmail.com", role = "ctb"),
        person("Giovanni", "Palla", email =
        "giov.pll@gmail.com", role = "ctb"),
        person("Maksim", "Kholmatov", email =
        "maksim.kholmatov@embl.de", role = "ctb"))
Description: Genetic variants associated with diseases often affect non-coding regions, thus likely having a regulatory role. To understand the effects of genetic variants in these regulatory regions, identifying genes that are modulated by specific regulatory elements (REs) is crucial. The effect of gene regulatory elements, such as enhancers, is often cell-type specific, likely because the combinations of transcription factors (TFs) that are regulating a given enhancer have cell-type specific activity. This TF activity can be quantified with existing tools such as diffTF and captures differences in binding of a TF in open chromatin regions. Collectively, this forms a gene regulatory network (GRN) with cell-type and data-specific TF-RE and RE-gene links. Here, we reconstruct such a GRN using single-cell or bulk RNAseq and open chromatin (e.g., using ATACseq or ChIPseq for open chromatin marks) and optionally (Capture) Hi-C data. Our network contains different types of links, connecting TFs to regulatory elements, the latter of which is connected to genes in the vicinity or within the same chromatin domain (TAD). We use a statistical framework to assign empirical FDRs and weights to all links using a permutation-based approach.
Imports: futile.logger, checkmate, patchwork (>= 1.2.0), reshape2,
        data.table, matrixStats, Matrix, GenomicRanges, RColorBrewer,
        ComplexHeatmap, DESeq2, circlize, progress, utils, methods,
        stringr, tools, scales, igraph, S4Vectors, ggplot2, rlang,
        Biostrings, GenomeInfoDb (>= 1.34.8), SummarizedExperiment,
        forcats, gridExtra, limma, tidyselect, readr, grid, tidyr (>=
        1.3.0), dplyr, stats, grDevices, graphics, magrittr, tibble,
        viridis, colorspace, biomaRt, topGO, AnnotationHub, ensembldb
Depends: R (>= 4.2.0)
Suggests: knitr, BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Hsapiens.UCSC.hg38, BSgenome.Mmusculus.UCSC.mm39,
        BSgenome.Mmusculus.UCSC.mm10, BSgenome.Mmusculus.UCSC.mm9,
        BSgenome.Rnorvegicus.UCSC.rn6, BSgenome.Rnorvegicus.UCSC.rn7,
        BSgenome.Dmelanogaster.UCSC.dm6,
        BSgenome.Mmulatta.UCSC.rheMac10,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene,
        TxDb.Mmusculus.UCSC.mm39.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene,
        TxDb.Mmusculus.UCSC.mm9.knownGene,
        TxDb.Rnorvegicus.UCSC.rn6.refGene,
        TxDb.Rnorvegicus.UCSC.rn7.refGene,
        TxDb.Dmelanogaster.UCSC.dm6.ensGene,
        TxDb.Mmulatta.UCSC.rheMac10.refGene, org.Hs.eg.db,
        org.Mm.eg.db, org.Rn.eg.db, org.Dm.eg.db, org.Mmu.eg.db, IHW,
        clusterProfiler, ReactomePA, DOSE, BiocFileCache, ChIPseeker,
        testthat (>= 3.0.0), BiocStyle, csaw, BiocParallel, WGCNA,
        variancePartition, purrr, EDASeq, JASPAR2022, JASPAR2024,
        RSQLite, TFBSTools, motifmatchr, rbioapi, LDlinkR
VignetteBuilder: knitr
biocViews: Software, GeneExpression, GeneRegulation, NetworkInference,
        GeneSetEnrichment, BiomedicalInformatics, Genetics,
        Transcriptomics, ATACSeq, RNASeq, GraphAndNetwork, Regression,
        Transcription, ChIPSeq
License: Artistic-2.0
URL: https://grp-zaugg.embl-community.io/GRaNIE
BugReports: https://git.embl.de/grp-zaugg/GRaNIE/issues
RoxygenNote: 7.3.1
Config/testthat/parallel: true
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/GRaNIE
git_branch: devel
git_last_commit: 43246b6
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-07
NeedsCompilation: no
Packaged: 2025-11-07 22:56:48 UTC; biocbuild
Author: Christian Arnold [cre, aut],
  Judith Zaugg [aut],
  Rim Moussa [aut],
  Armando Reyes-Palomares [ctb],
  Giovanni Palla [ctb],
  Maksim Kholmatov [ctb]
Maintainer: Christian Arnold <chrarnold@web.de>
