# Generated by roxygen2: do not edit by hand

export(AR_classification_wrapper)
export(addConnections_TF_peak)
export(addConnections_peak_gene)
export(addData)
export(addSNPData)
export(addTFBS)
export(add_TF_gene_correlation)
export(add_featureVariation)
export(build_eGRN_graph)
export(calculateCommunitiesEnrichment)
export(calculateCommunitiesStats)
export(calculateGeneralEnrichment)
export(calculateTFEnrichment)
export(changeOutputDirectory)
export(deleteIntermediateData)
export(filterConnectionsForPlotting)
export(filterData)
export(filterGRNAndConnectGenes)
export(generateStatsSummary)
export(getCounts)
export(getGRNConnections)
export(getGRNSummary)
export(getParameters)
export(getTopNodes)
export(initializeGRN)
export(loadExampleObject)
export(nGenes)
export(nPeaks)
export(nTFs)
export(overlapPeaksAndTFBS)
export(performAllNetworkAnalyses)
export(plotCommunitiesEnrichment)
export(plotCommunitiesStats)
export(plotCorrelations)
export(plotDiagnosticPlots_TFPeaks)
export(plotDiagnosticPlots_TFPeaks_GC)
export(plotDiagnosticPlots_peakGene)
export(plotGeneralEnrichment)
export(plotGeneralGraphStats)
export(plotPCA_all)
export(plotTFEnrichment)
export(plot_stats_connectionSummary)
export(visualizeGRN)
import(GenomeInfoDb)
import(GenomicRanges)
import(checkmate)
import(ggplot2)
import(grDevices)
import(graphics)
import(patchwork)
import(tibble)
import(tidyr)
import(utils)
importFrom(GenomicRanges,mcols)
importFrom(Matrix,Matrix)
importFrom(biomaRt,getBM)
importFrom(biomaRt,useEnsembl)
importFrom(circlize,colorRamp2)
importFrom(ensembldb,genes)
importFrom(ggplot2,`%+%`)
importFrom(grid,gpar)
importFrom(magrittr,`%>%`)
importFrom(methods,is)
importFrom(methods,new)
importFrom(rlang,.data)
importFrom(rlang,`:=`)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tools,R_user_dir)
importFrom(utils,packageVersion)
importFrom(utils,tail)
