#' @param normalization Character. Default \code{cyclicLoess}. One of \code{cyclicLoess}, \code{sizeFactors}, \code{quantile}, or \code{none}. 
#' Normalization procedure. When set to \code{cyclicLoess}, the \code{csaw} package is required (as it is listed under \code{Suggests}, it may not be installed).
