#' @param outputFolder Character or \code{NULL}. Default \code{NULL}. If set to \code{NULL}, the default output folder as specified when initiating the 
#' object in \code{\link{initializeGRN}} will be used. Otherwise, all output from this function will be put into the specified folder. 
#' If a folder is provided, while we recommend specifying an absolute path, a relative one also works.
