#' @param addRobustRegression \code{TRUE} or \code{FALSE}. EXPERIMENTAL. Default \code{FALSE}. 
#' Use a robust regression in addition to a non-robust one? Significantly increases overall running time.
#' If set to \code{TRUE}, the package \code{robust} is required (as it is listed under \code{Suggests}, it may not be installed).
